@@ -759,11 +759,16 @@ static void sStatment_copy_deeply_stack(sStatment* dest, sStatment* source)
 
 void sStatment_delete(sStatment* self)
 {
-    int i;
-    for(i=0; i<self->mCommandsNum; i++) {
-        sCommand_delete(self->mCommands + i);
+    if(self->mFlags & STATMENT_FLAGS_KIND_NODETREE) {
+        sNodeTree_free(self->mNodeTree);
+    }
+    else {
+        int i;
+        for(i=0; i<self->mCommandsNum; i++) {
+            sCommand_delete(self->mCommands + i);
+        }
+        if(self->mFName) FREE(self->mFName);
     }
-    if(self->mFName) FREE(self->mFName);
 }
 
 sObject* block_new_on_gc(BOOL user_object)
