@@ -22,7 +22,7 @@
 #include <ncurses/ncurses.h>
 #endif
 
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 
 BOOL cmd_exit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
@@ -127,7 +127,7 @@ BOOL cmd_if(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 BOOL cmd_return(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(runinfo->mArgsNumRuntime >= 2) {
-        uchar code = atoi(runinfo->mArgsRuntime[1]);
+        unsigned char code = atoi(runinfo->mArgsRuntime[1]);
         runinfo->mRCode = RCODE_RETURN | code;
 
         return FALSE;
@@ -311,6 +311,54 @@ BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+/*
+BOOL cmd_sweep(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    if(runinfo->mArgsNumRuntime == 1) {
+        int num = gc_sweep();
+
+        char buf[BUFSIZ];
+        int size = snprintf(buf, BUFSIZ, "%d objects deleted\n", num);
+
+        if(!fd_write(nextout, buf, size)) {
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+            return FALSE;
+        }
+        runinfo->mRCode = 0;
+    }
+    else {
+        int i;
+        for(i=1; i<runinfo->mArgsNumRuntime; i++) {
+            sObject* object;
+            sObject* name = STRING_NEW_STACK("");
+            if(!get_object_prefix_and_name_from_argument(&object, name, runinfo->mArgsRuntime[i], runinfo->mCurrentObject, runinfo->mRunningObject, runinfo)) {
+                return FALSE;
+            }
+
+            if(object && STYPE(object) == T_UOBJECT) {
+                sObject* item = uobject_item(object, string_c_str(name));
+
+                if(item) {
+                    if(!IS_USER_OBJECT(item)) {
+                        err_msg("it's not user object. can't delete", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                        return FALSE;
+                    }
+
+                    if(item != gRootObject) {
+                        if(uobject_erase(object, string_c_str(name))) {
+                            runinfo->mRCode = 0;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    return TRUE;
+}
+*/
+
 BOOL cmd_load(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     if(sRunInfo_option(runinfo, "-dynamic-library")) {
@@ -459,30 +507,34 @@ BOOL cmd_stackframe(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     char buf[BUFSIZ];
 
-    uobject_it* it = uobject_loop_begin(SFUN(runinfo->mRunningObject).mLocalObjects);
-    while(it) {
-        sObject* object2 = uobject_loop_item(it);
-        char* key = uobject_loop_key(it);
+    sObject* running_object = runinfo->mRunningObject;
 
-        char* obj_kind[T_TYPE_MAX] = {
-            NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object"
-        };
-        int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[STYPE(object2)]);
+    if(running_object&& (STYPE(running_object) == T_FUN || STYPE(running_object) == T_CLASS)) {
+        uobject_it* it = uobject_loop_begin(SFUN(running_object).mLocalObjects);
+        while(it) {
+            sObject* object2 = uobject_loop_item(it);
+            char* key = uobject_loop_key(it);
 
-        if(!fd_write(nextout, buf, size)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-            runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-            return FALSE;
+            char* obj_kind[T_TYPE_MAX] = {
+                NULL, "var", "array", "hash", "list", "native function", "block", "file dicriptor", "file dicriptor", "job", "object", "function", "class", "external program", "completion", "external object", "alias"
+            };
+            int size = snprintf(buf, BUFSIZ, "%s: %s\n", key, obj_kind[STYPE(object2)]);
+
+            if(!fd_write(nextout, buf, size)) {
+                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                return FALSE;
+            }
+
+            it = uobject_loop_next(it);
         }
 
-        it = uobject_loop_next(it);
-    }
-
-    if(uobject_count(SFUN(runinfo->mRunningObject).mLocalObjects) == 0) {
-        runinfo->mRCode = 1;
-    }
-    else {
-        runinfo->mRCode = 0;
+        if(uobject_count(SFUN(runinfo->mRunningObject).mLocalObjects) == 0) {
+            runinfo->mRCode = 1;
+        }
+        else {
+            runinfo->mRCode = 0;
+        }
     }
 
     return TRUE;
@@ -753,64 +805,66 @@ BOOL cmd_ablock(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sObject* fun = runinfo->mRunningObject;
 
-    if(sRunInfo_option(runinfo, "-number")) {
-        if(SFUN(fun).mArgBlocks) {
-            char buf[32];
-            int size = snprintf(buf, 32, "%d\n", vector_count(SFUN(fun).mArgBlocks));
+    if(fun && (STYPE(fun) == T_CLASS || STYPE(fun) == T_FUN)) {
+        if(sRunInfo_option(runinfo, "-number")) {
+            if(SFUN(fun).mArgBlocks) {
+                char buf[32];
+                int size = snprintf(buf, 32, "%d\n", vector_count(SFUN(fun).mArgBlocks));
 
-            if(!fd_write(nextout, buf, size))
-            {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                return FALSE;
+                if(!fd_write(nextout, buf, size))
+                {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+
+                runinfo->mRCode = 0;
+            }
+        }
+        else if(sRunInfo_option(runinfo, "-run")) {
+            int block_num;
+            if(runinfo->mArgsNumRuntime > 1) {
+                block_num = atoi(runinfo->mArgsRuntime[1]);
+            }
+            else {
+                block_num = 0;
             }
 
-            runinfo->mRCode = 0;
-        }
-    }
-    else if(sRunInfo_option(runinfo, "-run")) {
-        int block_num;
-        if(runinfo->mArgsNumRuntime > 1) {
-            block_num = atoi(runinfo->mArgsRuntime[1]);
+            if(SFUN(fun).mArgBlocks && block_num < vector_count(SFUN(fun).mArgBlocks)) {
+                sObject* block = vector_item(SFUN(fun).mArgBlocks, block_num);
+
+                int rcode = 0;
+                if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) 
+                {
+                    err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = rcode;
+                    return FALSE;
+                }
+
+                runinfo->mRCode = 0;
+            }
         }
         else {
-            block_num = 0;
-        }
-
-        if(SFUN(fun).mArgBlocks && block_num < vector_count(SFUN(fun).mArgBlocks)) {
-            sObject* block = vector_item(SFUN(fun).mArgBlocks, block_num);
-
-            int rcode = 0;
-            if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) 
-            {
-                err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                runinfo->mRCode = rcode;
-                return FALSE;
+            int block_num;
+            if(runinfo->mArgsNumRuntime > 1) {
+                block_num = atoi(runinfo->mArgsRuntime[1]);
+            }
+            else {
+                block_num = 0;
             }
 
-            runinfo->mRCode = 0;
-        }
-    }
-    else {
-        int block_num;
-        if(runinfo->mArgsNumRuntime > 1) {
-            block_num = atoi(runinfo->mArgsRuntime[1]);
-        }
-        else {
-            block_num = 0;
-        }
+            if(SFUN(fun).mArgBlocks && block_num < vector_count(SFUN(fun).mArgBlocks)) {
+                sObject* block = vector_item(SFUN(fun).mArgBlocks, block_num);
 
-        if(SFUN(fun).mArgBlocks && block_num < vector_count(SFUN(fun).mArgBlocks)) {
-            sObject* block = vector_item(SFUN(fun).mArgBlocks, block_num);
+                if(!fd_write(nextout, SBLOCK(block).mSource, strlen(SBLOCK(block).mSource)))
+                {
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
 
-            if(!fd_write(nextout, SBLOCK(block).mSource, strlen(SBLOCK(block).mSource)))
-            {
-                err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                return FALSE;
+                runinfo->mRCode = 0;
             }
-
-            runinfo->mRCode = 0;
         }
     }
 
@@ -947,3 +1001,5 @@ BOOL cmd_fg(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
     return TRUE;
 }
+
+
