@@ -23,7 +23,7 @@
 #include <ncurses/ncurses.h>
 #endif
 
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 
 BOOL cmd_condition_n(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
