@@ -1,5 +1,5 @@
 #include "config.h"
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 #include <errno.h>
 #include <time.h>
 #include <stdlib.h>
@@ -31,10 +31,12 @@ static void str_cut(enum eKanjiCode code, char* mbs, int termsize, char* dest_mb
         wchar_t* wcs_tmp;
         int i;
 
-        wcs = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1));
-        wcs_tmp = (wchar_t*)MALLOC(sizeof(wchar_t)*(termsize+1));
-        if(mbstowcs(wcs, mbs, termsize+1) == -1) {
-            mbstowcs(wcs, "?????", termsize+1);
+        const int len = strlen(mbs) + 1;
+
+        wcs = (wchar_t*)MALLOC(sizeof(wchar_t)*len);
+        wcs_tmp = (wchar_t*)MALLOC(sizeof(wchar_t)*len);
+        if(mbstowcs(wcs, mbs, len) == -1) {
+            mbstowcs(wcs, "?????", len);
         }
 
         for(i=0; i<wcslen(wcs); i++) {
@@ -78,7 +80,7 @@ static void selector_view(BOOL p_view, int scrolltop, int cursor, sObject* nexti
     const int maxx = mgetmaxx();
     const int maxy = mgetmaxy();
 
-    clear();
+    mclear();
     *n = scrolltop;
     const int start_line = *n;
     const int end_line = *n + maxy-1;
@@ -117,7 +119,7 @@ static void selector_view(BOOL p_view, int scrolltop, int cursor, sObject* nexti
                 mvprintw(y, 0, "%s", one_line);
             }
             else {
-                mvprintw(y, 0, head);
+                mvprintw(y, 0, "%s", head);
                 printw("%s", one_line);
             }
             y++;
@@ -227,9 +229,46 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sObject* search_str = STRING_NEW_STACK("");
 
     if(runinfo->mFilter) {
-        fd_split(nextin, lf, TRUE, FALSE, FALSE);
+        sObject* nextin2 = FD_NEW_STACK();
 
-        if(vector_count(SFD(nextin).mLines) > 0) {
+        /// make control characters visible ///
+        char* p = SFD(nextin).mBuf;
+        while(p - SFD(nextin).mBuf < SFD(nextin).mBufLen) {
+            if(*p == '\n') {
+                if(!fd_writec(nextin2, *p)) {
+                    err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+                p++;
+            }
+            else if(*p >= 0 && *p <= 31) {
+                if(!fd_writec(nextin2, '^')) {
+                    err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+                if(!fd_writec(nextin2, *p+'@')) {
+                    err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+                p++;
+            }
+            else {
+                if(!fd_writec(nextin2, *p)) {
+                    err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
+                    return FALSE;
+                }
+                p++;
+            }
+        }
+
+        /// go ///
+        fd_split(nextin2, lf, FALSE, FALSE, FALSE);
+
+        if(vector_count(SFD(nextin2).mLines) > 0) {
             msave_ttysettings();
             msave_screen();
             initscr();
@@ -239,8 +278,8 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             const int maxx = mgetmaxx();
             const int maxy = mgetmaxy();
 
-            int* markfiles = MALLOC(sizeof(int)*vector_count(SFD(nextin).mLines));
-            memset(markfiles, 0, sizeof(int)*vector_count(SFD(nextin).mLines));
+            int* markfiles = MALLOC(sizeof(int)*vector_count(SFD(nextin2).mLines));
+            memset(markfiles, 0, sizeof(int)*vector_count(SFD(nextin2).mLines));
 
             static int scrolltop = 0;
             static int cursor = 0;
@@ -253,15 +292,15 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             if(cursor < 0) {
                 cursor = 0;
             }
-            if(cursor >= vector_count(SFD(nextin).mLines)) {
-                cursor = vector_count(SFD(nextin).mLines)-1;
+            if(cursor >= vector_count(SFD(nextin2).mLines)) {
+                cursor = vector_count(SFD(nextin2).mLines)-1;
                 if(cursor < 0) cursor = 0;
             }
             if(cursor < scrolltop) {
                 int i = cursor;
                 int width_sum = 0;
                 while(width_sum < maxy) {
-                    char* line = vector_item(SFD(nextin).mLines, i);
+                    char* line = vector_item(SFD(nextin2).mLines, i);
                     int width = str_termlen(code, line) / maxx + 1;
                     width_sum += width;
                     i--;
@@ -276,8 +315,9 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             while(1) {
                 /// view ///
+                mclear();
                 int n;
-                selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &n);
+                selector_view(FALSE, scrolltop, cursor, nextin2, markfiles, code, &n);
                 refresh();
 
                 /// input ///
@@ -308,7 +348,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     cursor -= (maxy -1);
                 }
                 else if(key == 12) {
-                    clear();
+                    mclear();
                     refresh();
                 }
                 else if(key == 'g') {
@@ -324,7 +364,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 }
                 else if(key == 'a' && multiple) {
                     int i;
-                    for(i=0; i<vector_count(SFD(nextin).mLines); i++) {
+                    for(i=0; i<vector_count(SFD(nextin2).mLines); i++) {
                         markfiles[i] = !markfiles[i];
                     }
                 }
@@ -333,7 +373,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                     while(1) {
                         int dummy;
-                        selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &dummy);
+                        selector_view(FALSE, scrolltop, cursor, nextin2, markfiles, code, &dummy);
                         const int maxy = mgetmaxy();
                         mclear_online(maxy-1);
                         mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
@@ -351,27 +391,27 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         }
                     }
 
-                    selector_search_forward(nextin, string_c_str(search_str), &cursor);
+                    selector_search_forward(nextin2, string_c_str(search_str), &cursor);
                 }
                 else if(key == 'n') {
                     int dummy;
-                    selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &dummy);
+                    selector_view(FALSE, scrolltop, cursor, nextin2, markfiles, code, &dummy);
                     const int maxy = mgetmaxy();
                     mclear_online(maxy-1);
                     mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
                     refresh();
 
-                    selector_search_forward(nextin, string_c_str(search_str), &cursor);
+                    selector_search_forward(nextin2, string_c_str(search_str), &cursor);
                 }
                 else if(key == 'N') {
                     int dummy;
-                    selector_view(FALSE, scrolltop, cursor, nextin, markfiles, code, &dummy);
+                    selector_view(FALSE, scrolltop, cursor, nextin2, markfiles, code, &dummy);
                     const int maxy = mgetmaxy();
                     mclear_online(maxy-1);
                     mvprintw(maxy-1, 0, "/%s", string_c_str(search_str));
                     refresh();
 
-                    selector_search_backward(nextin, string_c_str(search_str), &cursor);
+                    selector_search_backward(nextin2, string_c_str(search_str), &cursor);
                 }
                 else if(key == ' ' && multiple) {
                     markfiles[cursor] = !markfiles[cursor];
@@ -381,9 +421,9 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     if(multiple) {
                         BOOL flg_mark = FALSE;
                         int k;
-                        for(k=0; k<vector_count(SFD(nextin).mLines); k++) {
+                        for(k=0; k<vector_count(SFD(nextin2).mLines); k++) {
                             if(markfiles[k]) {
-                                char* str = vector_item(SFD(nextin).mLines, k);
+                                char* str = vector_item(SFD(nextin2).mLines, k);
                                 if(!fd_write(nextout,str, strlen(str))) {
                                     err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -396,7 +436,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         }
 
                         if(!flg_mark) {
-                            char* str = vector_item(SFD(nextin).mLines, cursor);
+                            char* str = vector_item(SFD(nextin2).mLines, cursor);
                             if(!fd_write(nextout, str, strlen(str))) {
                                 err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -406,7 +446,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         }
                     }
                     else {
-                        char* str = vector_item(SFD(nextin).mLines, cursor);
+                        char* str = vector_item(SFD(nextin2).mLines, cursor);
                         if(!fd_write(nextout, str, strlen(str))) {
                             err_msg("interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -414,7 +454,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                             return FALSE;
                         }
                     }
-                    if(SFD(nextin).mBufLen == 0) {
+                    if(SFD(nextin2).mBufLen == 0) {
                         runinfo->mRCode = RCODE_NFUN_NULL_INPUT;
                     }
                     else {
@@ -426,8 +466,8 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 if(cursor < 0) {
                     cursor = 0;
                 }
-                if(cursor >= vector_count(SFD(nextin).mLines)) {
-                    cursor = vector_count(SFD(nextin).mLines)-1;
+                if(cursor >= vector_count(SFD(nextin2).mLines)) {
+                    cursor = vector_count(SFD(nextin2).mLines)-1;
                     if(cursor < 0) cursor = 0;
                 }
                 if(cursor >= n) {
@@ -437,7 +477,7 @@ BOOL cmd_selector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     int i = cursor;
                     int width_sum = 0;
                     while(width_sum < (maxy-1)) {
-                        char* line = vector_item(SFD(nextin).mLines, i);
+                        char* line = vector_item(SFD(nextin2).mLines, i);
                         int width = str_termlen(code, line) / maxx + 1;
                         width_sum += width;
                         i--;
@@ -592,7 +632,7 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         while(1) {
             /// view ///
-            clear();
+            mclear();
             int n;
             selector_view(TRUE, scrolltop, cursor, nextin2, markfiles, code, &n);
             refresh();
@@ -625,14 +665,14 @@ BOOL cmd_p(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 cursor -= (maxy -1);
             }
             else if(key == 12) {
-                clear();
+                mclear();
                 refresh();
             }
             else if(key == 'g') {
                 cursor = 0;
             }
             else if(key == 12) {
-                clear();
+                mclear();
                 refresh();
             }
             else if(key == '/') {
@@ -1164,7 +1204,7 @@ BOOL cmd_fselector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int cursor = 0;
         while(1) {
             /// 描写 ///
-            clear();
+            mclear();
             mvprintw(0,0,"%s", current_dir);
 
             int page = cursor/((maxy-1)*4);
@@ -1235,7 +1275,7 @@ BOOL cmd_fselector(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 cursor -= (maxy-1)*4;
             }
             else if(key == 12) {
-                clear();
+                mclear();
                 refresh();
             }
             else if(key == ' ' && multiple) {
