@@ -380,10 +380,11 @@ BOOL sCommand_expand_env(sCommand* command, sObject* object, sObject* nextin, sR
                     return FALSE;
                 }
 
+
                 int j;
                 for(j=0; j<strings->mNumber; j++) {
                     char* arg = strings->mStrings[j];
-                    if(object && (STYPE(object) == T_NFUN && nfun_option_with_argument(object, arg) || STYPE(object) == T_CLASS && class_option_with_argument(object,arg) || STYPE(object) == T_FUN && fun_option_with_argument(object, arg) ))
+                    if(object && (STYPE(object) == T_NFUN && nfun_option_with_argument(object, arg) || STYPE(object) == T_CLASS && class_option_with_argument(object,arg) || STYPE(object) == T_FUN && fun_option_with_argument(object, arg)))
                     {
                         if(j + 1 >= strings->mNumber) {
                             char buf[BUFSIZ];
@@ -409,12 +410,20 @@ BOOL sCommand_expand_env(sCommand* command, sObject* object, sObject* nextin, sR
                         j++;
                     }
                     else {
-                        if(!sRunInfo_put_option(runinfo, MANAGED STRDUP(arg))) {
-                            char buf[BUFSIZ];
-                            snprintf(buf, BUFSIZ, "option number max (%s)\n", arg);
-                            err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                            sString_delete(strings);
-                            return FALSE;
+                        if(external_command) {
+                            sRunInfo_add_arg(runinfo, MANAGED strings->mStrings[j]);
+                            strings->mStrings[j] = NULL;
+                        }
+                        else {
+                            if(!sRunInfo_put_option(runinfo, MANAGED arg)) {
+                                char buf[BUFSIZ];
+                                snprintf(buf, BUFSIZ, "option number max (%s)\n", arg);
+                                err_msg(buf, runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                                sString_delete(strings);
+                                return FALSE;
+                            }
+
+                            strings->mStrings[j] = NULL;
                         }
                     }
                 }
