@@ -318,6 +318,27 @@ static BOOL read_backquote(char** p, sBuf* buf, char* sname, int* sline, sComman
     return TRUE;
 }
 
+static BOOL colon_statment(char** p, ALLOC sNodeTree** node, char* sname, int* sline)
+{
+    *node = NULL;
+    while(**p) {
+        if(**p == ')' || **p == ']' || **p == '`' || **p == '}') {
+            break;
+        }
+        if(**p == '\n') {
+            (*p)++;
+            break;
+        }
+        else {
+            if(!expression_add_sub(ALLOC node, p, sname, sline)) {
+                sNodeTree_free(*node);
+                return FALSE;
+            }
+        }
+    }
+    return TRUE;
+}
+
 static BOOL add_argument_to_command(MANAGED sBuf* buf, sCommand* command, char* sname, int* sline);
 
 static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCommand* command, sObject* block, sObject** current_object, BOOL read_until_bracket_close, BOOL read_until_backquote)
@@ -438,9 +459,6 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, sCom
                 }
             }
         }
-        /// backslash quote ///
-        else if(read_until_backquote && **p == '\\' && *(*p+1) =='`') {
-        }
         /// quote ///
         else if(**p == '\\') {
             if(!read_backslash(p, buf, sname, sline)) {
@@ -1028,157 +1046,171 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
 /// FALSE: indicate to occure error
 static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object, BOOL read_until_backquote)
 {
-    /// a reversing code ///
-    if(**p == '!') {
-        (*p)++;
+/*
+    /// expression ///
+    if(**p == ':' && *(*p+1) != ':') {
+        p++;
         skip_spaces(p);
 
-        statment->mFlags |= STATMENT_FLAGS_REVERSE;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_HEAD;
-    }
-
-    /// a global pipe in ///
-    if(**p == '|' && *(*p+1) == '>') {
-        (*p)+=2;
-
-        skip_spaces(p);
-
-        statment->mFlags |= STATMENT_FLAGS_GLOBAL_PIPE_IN;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_HEAD;
-    }
-    /// a context pipe ///
-    else if(**p == '|' && *(*p+1) != '|') {
-        (*p)++;
-
-        char buf[16];
-        char* p2 = buf;
-        while(**p == '-' || **p >= '0' && **p <= '9') {
-            *p2++ = **p;
-            (*p)++;
-        }
-        *p2 = 0;
-
-        skip_spaces(p);
-
-        statment->mFlags |= STATMENT_FLAGS_CONTEXT_PIPE;
-        statment->mFlags |= atoi(buf) & STATMENT_FLAGS_CONTEXT_PIPE_NUMBER;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_HEAD;
-    }
-
-    while(**p) {
-        if(statment->mCommandsNum >= STATMENT_COMMANDS_MAX) {
-            err_msg("Overflow command number. Please use global pipe.", sname, *sline, NULL);
+        sNodeTree* node;
+        if(!colon_statment(p, ALLOC &node, sname, sline)) {
             return FALSE;
         }
-
-        SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_ENV|COMPLETION_FLAGS_TILDA);
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_INPUTING_COMMAND_NAME;
-
-        sCommand* command = sCommand_new(statment);
-        if(!read_command(p, command, statment, block, sname, sline, current_object, read_until_backquote)) {
-            return FALSE;
-        }
-
-        SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_STATMENT_HEAD;
-
-        /// There is no argument, so delete last command in block ///
-        if(command->mArgsNum == 0)
-        {
-            if(command->mMessagesNum > 0) {
-                err_msg("require command name", sname, *sline, NULL);
-                return FALSE;
-            }
-            else {
-                sCommand_delete(command);
-                memset(command, 0, sizeof(sCommand));
-                statment->mCommandsNum--;
-            }
-        }
-
-        if(**p == '|' && *(*p+1) != '>' && *(*p+1) != '|') // chains a next command with pipe
-        {
+    }
+    else {
+*/
+        /// a reversing code ///
+        if(**p == '!') {
             (*p)++;
+            skip_spaces(p);
+
+            statment->mFlags |= STATMENT_FLAGS_REVERSE;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_HEAD;
+        }
+
+        /// a global pipe in ///
+        if(**p == '|' && *(*p+1) == '>') {
+            (*p)+=2;
 
             skip_spaces(p);
 
-            if(**p == 0) {
-                SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_COMMAND_END;
+            statment->mFlags |= STATMENT_FLAGS_GLOBAL_PIPE_IN;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_HEAD;
+        }
+        /// a context pipe ///
+        else if(**p == '|' && *(*p+1) != '|') {
+            (*p)++;
+
+            char buf[16];
+            char* p2 = buf;
+            while(**p == '-' || **p >= '0' && **p <= '9') {
+                *p2++ = **p;
+                (*p)++;
+            }
+            *p2 = 0;
+
+            skip_spaces(p);
+
+            statment->mFlags |= STATMENT_FLAGS_CONTEXT_PIPE;
+            statment->mFlags |= atoi(buf) & STATMENT_FLAGS_CONTEXT_PIPE_NUMBER;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_HEAD;
+        }
+
+        while(**p) {
+            if(statment->mCommandsNum >= STATMENT_COMMANDS_MAX) {
+                err_msg("Overflow command number. Please use global pipe.", sname, *sline, NULL);
+                return FALSE;
+            }
+
+            SBLOCK(block).mCompletionFlags &= ~(COMPLETION_FLAGS_ENV|COMPLETION_FLAGS_TILDA);
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_INPUTING_COMMAND_NAME;
+
+            sCommand* command = sCommand_new(statment);
+            if(!read_command(p, command, statment, block, sname, sline, current_object, read_until_backquote)) {
+                return FALSE;
+            }
+
+            SBLOCK(block).mCompletionFlags &= ~COMPLETION_FLAGS_STATMENT_HEAD;
+
+            /// There is no argument, so delete last command in block ///
+            if(command->mArgsNum == 0)
+            {
+                if(command->mMessagesNum > 0) {
+                    err_msg("require command name", sname, *sline, NULL);
+                    return FALSE;
+                }
+                else {
+                    sCommand_delete(command);
+                    memset(command, 0, sizeof(sCommand));
+                    statment->mCommandsNum--;
+                }
+            }
+
+            if(**p == '|' && *(*p+1) != '>' && *(*p+1) != '|') // chains a next command with pipe
+            {
+                (*p)++;
+
+                skip_spaces(p);
+
+                if(**p == 0) {
+                    SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_COMMAND_END;
+                    break;
+                }
+            }
+            else {
                 break;
             }
         }
-        else {
-            break;
+
+        /// global pipe out
+        if(**p == '|' && *(*p+1) == '>') {
+            (*p)+=2;
+
+            if(**p == '>') {
+                (*p)++;
+                statment->mFlags |= STATMENT_FLAGS_GLOBAL_PIPE_APPEND;
+            }
+            else {
+                statment->mFlags |= STATMENT_FLAGS_GLOBAL_PIPE_OUT;
+            }
+
+            skip_spaces(p);
         }
-    }
 
-    /// global pipe out
-    if(**p == '|' && *(*p+1) == '>') {
-        (*p)+=2;
+        /// the termination of a statment
+        if(**p == '&' && *(*p+1) == '&') {
+            (*p)+=2;
+            skip_spaces(p);
 
-        if(**p == '>') {
+            statment->mFlags |= STATMENT_FLAGS_ANDAND;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
+        }
+        else if(**p == '|' && *(*p+1) == '|') {
+            (*p)+=2;
+            skip_spaces(p);
+
+            statment->mFlags |= STATMENT_FLAGS_OROR;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
+        }
+        else if(**p == '&') {
             (*p)++;
-            statment->mFlags |= STATMENT_FLAGS_GLOBAL_PIPE_APPEND;
+            skip_spaces(p);
+
+            if(gAppType == kATOptC) {
+                err_msg("In script mode a job can't run on background. On interactive shell a job can run on background", sname, *sline, NULL);
+                return FALSE;
+            }
+
+            statment->mFlags |= STATMENT_FLAGS_BACKGROUND;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
+        }
+        else if(**p == '\n') {
+            (*p)++; 
+            skip_spaces(p);
+
+            statment->mFlags |= STATMENT_FLAGS_NORMAL;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
+            (*sline)++;
+        }
+        else if(**p == ';') {
+            (*p)++;
+            skip_spaces(p);
+
+            statment->mFlags |= STATMENT_FLAGS_NORMAL;
+            SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
+        }
+        else if(**p == 0) {
+            statment->mFlags |= STATMENT_FLAGS_NORMAL;
+        }
+        else if(**p == ')' || **p == '#' || read_until_backquote && **p == '`') {
         }
         else {
-            statment->mFlags |= STATMENT_FLAGS_GLOBAL_PIPE_OUT;
-        }
-
-        skip_spaces(p);
-    }
-
-    /// the termination of a statment
-    if(**p == '&' && *(*p+1) == '&') {
-        (*p)+=2;
-        skip_spaces(p);
-
-        statment->mFlags |= STATMENT_FLAGS_ANDAND;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
-    }
-    else if(**p == '|' && *(*p+1) == '|') {
-        (*p)+=2;
-        skip_spaces(p);
-
-        statment->mFlags |= STATMENT_FLAGS_OROR;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
-    }
-    else if(**p == '&') {
-        (*p)++;
-        skip_spaces(p);
-
-        if(gAppType == kATOptC) {
-            err_msg("In script mode a job can't run on background. On interactive shell a job can run on background", sname, *sline, NULL);
+            char buf[128];
+            snprintf(buf, 128, "unexpected token3 -->(%c)\n", **p);
+            err_msg(buf, sname, *sline, NULL);
             return FALSE;
         }
-
-        statment->mFlags |= STATMENT_FLAGS_BACKGROUND;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
-    }
-    else if(**p == '\n') {
-        (*p)++; 
-        skip_spaces(p);
-
-        statment->mFlags |= STATMENT_FLAGS_NORMAL;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
-        (*sline)++;
-    }
-    else if(**p == ';') {
-        (*p)++;
-        skip_spaces(p);
-
-        statment->mFlags |= STATMENT_FLAGS_NORMAL;
-        SBLOCK(block).mCompletionFlags |= COMPLETION_FLAGS_STATMENT_END;
-    }
-    else if(**p == 0) {
-        statment->mFlags |= STATMENT_FLAGS_NORMAL;
-    }
-    else if(**p == ')' || **p == '#' || read_until_backquote && **p == '`') {
-    }
-    else {
-        char buf[128];
-        snprintf(buf, 128, "unexpected token3 -->(%c)\n", **p);
-        err_msg(buf, sname, *sline, NULL);
-        return FALSE;
-    }
+//    }
 
     return TRUE;
 }
