@@ -17,7 +17,7 @@
 #include <signal.h>
 #include <dirent.h>
 #include <limits.h>
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 
 sObject* gStdin;
 sObject* gStdout;
@@ -51,10 +51,12 @@ BOOL add_object_to_objects_in_pipe(sObject* object, sRunInfo* runinfo, sCommand*
 {
     vector_add(gObjectsInPipe, object);
 
+/*
     if(vector_count(gObjectsInPipe) > XYZSH_OBJECTS_IN_PIPE_MAX) {
         err_msg("overflow objects in pipe", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
         return FALSE;
     }
+*/
 
     return TRUE;
 }
@@ -657,6 +659,15 @@ if(1) {
             if(gAppType != kATOptC) { (void)setpgid(pid, getpgid(getpid())); }
         }
 
+        /// set environment variables ////
+        char buf[128];
+
+        snprintf(buf, 128, "%d", mgetmaxx());
+        setenv("COLUMNS", buf, 1);
+
+        snprintf(buf, 128, "%d", mgetmaxy());
+        setenv("LINES", buf, 1);
+
         xyzsh_restore_signal_default();
         sigchld_block(0);
 
@@ -785,18 +796,23 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
         else {
             char buf[BUFSIZ];
             err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+
             runinfo->mRCode = rcode;
-            SFUN(fun).mLocalObjects = local_objects;
+
             (void)vector_pop_back(gStackFrames);
+
+            SFUN(fun).mLocalObjects = local_objects;
+            SFUN(fun).mArgBlocks = arg_blocks;
             gRunInfoOfRunningObject = runinfo2;
+
             return FALSE;
         }
     }
 
-    (void)vector_pop_back(gStackFrames);
-
     runinfo->mRCode = rcode;
 
+    (void)vector_pop_back(gStackFrames);
+
     SFUN(fun).mLocalObjects = local_objects;
     SFUN(fun).mArgBlocks = arg_blocks;
     gRunInfoOfRunningObject = runinfo2;
@@ -817,6 +833,60 @@ static BOOL run_block(sObject* block, sObject* nextin, sObject* nextout, sRunInf
     return TRUE;
 }
 
+static BOOL run_alias(sObject* alias, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sObject* block = block_clone_on_stack(SALIAS(alias).mBlock, T_BLOCK);
+
+    sCommand* last_command = NULL;
+    if(SBLOCK(block).mStatmentsNum > 0) {
+        sStatment* statment = SBLOCK(block).mStatments + SBLOCK(block).mStatmentsNum -1;
+
+        if(statment->mCommandsNum > 0) {
+            last_command = statment->mCommands + statment->mCommandsNum -1;
+        }
+    }
+
+    if(last_command) {
+        sCommand* command = runinfo->mCommand;
+
+        if(!sCommand_add_command_without_command_name(last_command, command, runinfo->mSName, runinfo->mSLine)) {
+            return FALSE;
+        }
+    }
+
+    sObject* local_objects = SALIAS(alias).mLocalObjects;
+    sRunInfo* runinfo2 = gRunInfoOfRunningObject;
+    gRunInfoOfRunningObject = runinfo;
+
+    sObject* stackframe = UOBJECT_NEW_GC(8, gXyzshObject, "_stackframe", FALSE);
+    vector_add(gStackFrames, stackframe);
+    //uobject_init(stackframe);
+    SALIAS(alias).mLocalObjects = stackframe;
+
+    sObject* arg_blocks = SALIAS(alias).mArgBlocks;
+    SALIAS(alias).mArgBlocks = VECTOR_NEW_GC(16, FALSE);
+
+    int rcode = 0;
+    if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, alias)) {
+        sCommand* command = runinfo->mCommand;
+        err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+
+        (void)vector_pop_back(gStackFrames);
+        SALIAS(alias).mLocalObjects = local_objects;
+        SALIAS(alias).mArgBlocks = arg_blocks;
+        gRunInfoOfRunningObject = runinfo2;
+
+        return FALSE;
+    }
+
+    (void)vector_pop_back(gStackFrames);
+    SALIAS(alias).mLocalObjects = local_objects;
+    SALIAS(alias).mArgBlocks = arg_blocks;
+    gRunInfoOfRunningObject = runinfo2;
+
+    return TRUE;
+}
+
 BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     switch(STYPE(object)) {
@@ -870,6 +940,12 @@ BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* ru
             }
             break;
 
+        case T_ALIAS:
+            if(!run_alias(object, nextin, nextout, runinfo)) {
+                return FALSE;
+            }
+            break;
+
         case T_STRING: {
             char* str = string_c_str(object);
             int size = strlen(str);
@@ -1118,8 +1194,17 @@ static BOOL inherit_method(sRunInfo* runinfo, sObject** object, sCommand* comman
 {
     sObject* running_object = runinfo->mRunningObject;
 
-    if(runinfo->mRunningObject) {
+    if(running_object) {
         switch(STYPE(running_object)) {
+        case T_ALIAS: {
+            sObject* parent = SALIAS(running_object).mParent;
+
+            if(parent) {
+                *object = parent;
+            }
+            }
+            break;
+
         case T_FUN:  {
             sObject* parent = SFUN(running_object).mParent;
 
@@ -1156,6 +1241,56 @@ static BOOL inherit_method(sRunInfo* runinfo, sObject** object, sCommand* comman
     return TRUE;
 }
 
+BOOL get_object_from_command_name(sCommand* command, sObject* current_object, sObject** object, sObject** reciever, sRunInfo* runinfo)
+{
+    *reciever = current_object;
+
+    /// message passing with object prefix ///
+    if(command->mMessagesNum > 0) {
+        /// search for object ///
+        if(command->mMessages[0][0] == 0) {
+            *object = gRootObject;
+        }
+        else {
+            *object = access_object(command->mMessages[0], reciever, runinfo->mRunningObject);
+        }
+
+        int i = 1;
+        while(i < command->mMessagesNum) {
+            if(*object && STYPE(*object) == T_UOBJECT) {
+                *reciever = *object;
+                *object = uobject_item(*object, command->mMessages[i++]);
+            }
+            else {
+                err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                return FALSE;
+            }
+        }
+
+        if(*object && STYPE(*object) == T_UOBJECT) {
+            *reciever = *object;
+            *object = uobject_item(*object, command->mArgs[0]);
+        }
+        else {
+            err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            return FALSE;
+        }
+    }
+    /// message passing without object prefix ///
+    else {
+        *object = access_object(command->mArgs[0], reciever, runinfo->mRunningObject);
+    }
+
+    /// inherit object ///
+    if(*object == gInheritMethod || *object == gInheritMethod2) {
+        if(!inherit_method(runinfo, object, command)) {
+            return FALSE;
+        }
+    }
+
+    return TRUE;
+}
+
 static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout, sRunInfo* runinfo, sObject* current_object)
 {
     /// head of statment ///
@@ -1247,119 +1382,48 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         int opened_fd = -1;
 
         if(command->mArgsNum > 0) {
-            /// message passing with object prefix ///
-            if(command->mMessagesNum > 0) {
-                /// search for object ///
-                sObject* reciever = current_object;
-                sObject* object;
-                if(command->mMessages[0][0] == 0) {
-                    object = gRootObject;
-                }
-                else {
-                    object = access_object(command->mMessages[0], &reciever, runinfo->mRunningObject);
-                }
-                int i = 1;
-                while(i < command->mMessagesNum) {
-                    if(object && STYPE(object) == T_UOBJECT) {
-                        reciever = object;
-                        object = uobject_item(object, command->mMessages[i++]);
-                    }
-                    else {
-                        err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                        sRunInfo_command_delete(runinfo);
-                        return FALSE;
-                    }
-                }
+            sObject* reciever;
+            sObject* object;
+            if(!get_object_from_command_name(command, current_object, &object, &reciever, runinfo)) {
+                sRunInfo_command_delete(runinfo);
+                return FALSE;
+            }
+            
+            /// expand env ///
+            if(!sCommand_expand_env(command, object, pipein, runinfo))
+            {
+                sRunInfo_command_delete(runinfo);
+                return FALSE;
+            }
 
-                if(object && STYPE(object) == T_UOBJECT) {
-                    reciever = object;
-                    object = uobject_item(object, command->mArgs[0]);
+            /// redirect ///
+            if(!ready_for_redirect(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
+                sRunInfo_command_delete(runinfo);
+                return FALSE;;
+            }
+            
+            ASSERT(STYPE(nextin) == T_FD);
 
-                    /// inherit object ///
-                    if(object == gInheritMethod) {
-                        if(!inherit_method(runinfo, &object, command)) {
-                            sRunInfo_command_delete(runinfo);
-                            return FALSE;
-                        }
-                    }
-
-                    /// expand env ///
-                    if(!sCommand_expand_env(command, object, pipein, runinfo))
-                    {
-                        sRunInfo_command_delete(runinfo);
-                        return FALSE;
-                    }
-
-                    /// redirect ///
-                    if(!ready_for_redirect(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
-                        sRunInfo_command_delete(runinfo);
-                        return FALSE;;
-                    }
-                    
-                    ASSERT(STYPE(nextin) == T_FD);
-
-                    /// run object ///
-                    if(object) {
-                        runinfo->mRecieverObject = reciever;
-                        if(!run_object(object, nextin, nextout, runinfo)) {
-                            sRunInfo_command_delete(runinfo);
-                            return FALSE;
-                        }
-                    }
-                    else {
-                        err_msg("there is not this object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                        sRunInfo_command_delete(runinfo);
-                        return FALSE;
-                    }
-                }
-                else {
-                    err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            /// run object ///
+            if(object) {
+                runinfo->mRecieverObject = reciever;
+                if(!run_object(object, nextin, nextout, runinfo)) {
                     sRunInfo_command_delete(runinfo);
                     return FALSE;
                 }
             }
-            /// message passing without object prefix ///
             else {
-                sObject* reciever = current_object;
-                sObject* object = access_object(command->mArgs[0], &reciever, runinfo->mRunningObject);
-
-                /// inherit method ///
-                if(object == gInheritMethod) {
-                    if(!inherit_method(runinfo, &object, command)) {
-                        sRunInfo_command_delete(runinfo);
-                        return FALSE;
-                    }
-                }
-
-                /// expand env ///
-                if(!sCommand_expand_env(command, object, pipein, runinfo))
-                {
-                    sRunInfo_command_delete(runinfo);
-                    return FALSE;
-                }
-
-                /// redirect ///
-                if(!ready_for_redirect(&nextout, &nextin, &opened_fd, pipeout, last_program, statment, command, runinfo)) {
-                    sRunInfo_command_delete(runinfo);
-                    return FALSE;;
-                }
-                
-                ASSERT(STYPE(nextin) == T_FD);
-
-                if(object) {
-                    /// run object ///
-                    runinfo->mRecieverObject = reciever;
-                    if(!run_object(object, nextin, nextout, runinfo)) {
+                /// run external object ///
+                if(command->mMessagesNum == 0) {
+                    if(!run_external_command(runinfo->mArgsRuntime[0], nextin, nextout, runinfo)) {
                         sRunInfo_command_delete(runinfo);
                         return FALSE;
                     }
                 }
                 else {
-                    /// run external object ///
-                    if(!run_external_command(runinfo->mArgsRuntime[0], nextin, nextout, runinfo)) {
-                        sRunInfo_command_delete(runinfo);
-                        return FALSE;
-                    }
+                    err_msg("there is not this object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    sRunInfo_command_delete(runinfo);
+                    return FALSE;
                 }
             }
         }
