@@ -2,7 +2,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include "xyzsh/xyzsh.h"
+#include "xyzsh.h"
 
 static int gStackPageSize;
 static sObject** gStack;
