@@ -2,7 +2,17 @@
 
 #include "xyzsh.h"
 #include "temulator.h"
+
+#if defined(__DARWIN__)
 #include <util.h>
+#elif defined(__LINUX__)
+#define _XOPEN_SOURCE
+#define __USE_XOPEN
+#include <wchar.h>
+#include <pty.h>
+#include <ctype.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
