@@ -217,19 +217,29 @@ typedef struct {
 #define STATMENT_FLAGS_NORMAL 0x200000
 #define STATMENT_FLAGS_OROR 0x400000
 #define STATMENT_FLAGS_ANDAND 0x800000
+#define STATMENT_FLAGS_KIND_NODETREE 0x1000000
 #define STATMENT_FLAGS_CONTEXT_PIPE_NUMBER 0xFFFF
 
 #define STATMENT_COMMANDS_MAX 12
 #define XYZSH_OPTION_MAX 16
 
+struct _sNodeTree;
+
+typedef struct _sNodeTree sNodeTree;
+
 typedef struct {
-    sCommand mCommands[STATMENT_COMMANDS_MAX];
-    unsigned char mCommandsNum;
-
-    char* mFName;
-    ushort mLine;
-
     uint mFlags;
+
+    union {
+        struct {
+            sCommand mCommands[STATMENT_COMMANDS_MAX];
+            unsigned char mCommandsNum;
+
+            char* mFName;
+            ushort mLine;
+        };
+        struct _sNodeTree* mNodeTree;
+    };
 } sStatment;
 
 #define COMPLETION_FLAGS_BLOCK_OR_ENV_NUM 0xff
@@ -1063,6 +1073,28 @@ BOOL get_object_from_argument(sObject** object, char* str, sObject* current_obje
 BOOL get_object_prefix_and_name_from_argument(sObject** object, sObject* name, char* str, sObject* current_object, sObject* running_object, sRunInfo* runinfo) ;
 BOOL get_object_from_command_name(sCommand* command, sObject* current_object, sObject** object, sObject** reciever, sRunInfo* runinfo);
 
+enum eOperand { 
+    kOpAdd, kOpSub, kOpMult, kOpDiv
+};
+
+struct _sNodeTree {
+    unsigned char mType;   // 0: operand 1: value
+
+    union {
+        enum eOperand mOperand;
+        int mValue;
+    };
+
+    struct _sNodeTree* mLeft;
+    struct _sNodeTree* mRight;
+};
+
+sNodeTree* sNodeTree_create_operand(enum eOperand operand, sNodeTree* left, sNodeTree* right);
+sNodeTree* sNodeTree_create_value(int value, sNodeTree* left, sNodeTree* right);
+void sNodeTree_free(sNodeTree* self);
+
+BOOL expression_add_sub(sNodeTree** node, char** p, char* sname, int* sline);
+
 //////////////////////////////////////////////////////////////////////
 // xyzsh API
 //////////////////////////////////////////////////////////////////////
